package sun.awt.geom;

import java.awt.geom.Rectangle2D;
import java.awt.geom.IllegalPathStateException;
import java.awt.geom.PathIterator;
import java.util.Vector;

public abstract class Curve
{
    public static final int INCREASING = 1;
    public static final int DECREASING = -1;
    protected int direction;
    public static final int RECT_INTERSECTS = Integer.MIN_VALUE;
    public static final double TMIN = 0.001;
    
    public static void insertMove(Vector vector, double n, double n2) {
        vector.add(new Order0(n, n2));
    }
    
    public static void insertLine(Vector vector, double n, double n2, double n3, double n4) {
        if (n2 < n4) {
            vector.add(new Order1(n, n2, n3, n4, 1));
        }
        else if (n2 > n4) {
            vector.add(new Order1(n3, n4, n, n2, -1));
        }
    }
    
    public static void insertQuad(Vector vector, double n, double n2, double[] array) {
        double n3 = array[3];
        if (n2 > n3) {
            Order2.insert(vector, array, array[2], n3, array[0], array[1], n, n2, -1);
        }
        else {
            if (n2 == n3 && n2 == array[1]) {
                return;
            }
            Order2.insert(vector, array, n, n2, array[0], array[1], array[2], n3, 1);
        }
    }
    
    public static void insertCubic(Vector vector, double n, double n2, double[] array) {
        double n3 = array[5];
        if (n2 > n3) {
            Order3.insert(vector, array, array[4], n3, array[2], array[3], array[0], array[1], n, n2, -1);
        }
        else {
            if (n2 == n3 && n2 == array[1] && n2 == array[3]) {
                return;
            }
            Order3.insert(vector, array, n, n2, array[0], array[1], array[2], array[3], array[4], n3, 1);
        }
    }
    
    public static int pointCrossingsForPath(PathIterator pathIterator, double n, double n2) {
        if (pathIterator.isDone()) {
            return 0;
        }
        double[] array = new double[6];
        if (pathIterator.currentSegment(array) != 0) {
            throw new IllegalPathStateException("missing initial moveto in path definition");
        }
        pathIterator.next();
        double n3 = array[0];
        double n4 = array[1];
        int n5 = 0;
        while (!pathIterator.isDone()) {
            switch (pathIterator.currentSegment(array)) {
                case 0: {
                    if (n4 != n4) {
                        n5 += pointCrossingsForLine(n, n2, n3, n4, n3, n4);
                    }
                    n3 = (n3 = array[0]);
                    n4 = (n4 = array[1]);
                    break;
                }
                case 1: {
                    double n6 = array[0];
                    double n7 = array[1];
                    n5 += pointCrossingsForLine(n, n2, n3, n4, n6, n7);
                    n3 = n6;
                    n4 = n7;
                    break;
                }
                case 2: {
                    double n8 = array[2];
                    double n9 = array[3];
                    n5 += pointCrossingsForQuad(n, n2, n3, n4, array[0], array[1], n8, n9, 0);
                    n3 = n8;
                    n4 = n9;
                    break;
                }
                case 3: {
                    double n10 = array[4];
                    double n11 = array[5];
                    n5 += pointCrossingsForCubic(n, n2, n3, n4, array[0], array[1], array[2], array[3], n10, n11, 0);
                    n3 = n10;
                    n4 = n11;
                    break;
                }
                case 4: {
                    if (n4 != n4) {
                        n5 += pointCrossingsForLine(n, n2, n3, n4, n3, n4);
                    }
                    break;
                }
            }
            pathIterator.next();
        }
        if (n4 != n4) {
            n5 += pointCrossingsForLine(n, n2, n3, n4, n3, n4);
        }
        return n5;
    }
    
    public static int pointCrossingsForLine(double n, double n2, double n3, double n4, double n5, double n6) {
        if (n2 < n4 && n2 < n6) {
            return 0;
        }
        if (n2 >= n4 && n2 >= n6) {
            return 0;
        }
        if (n >= n3 && n >= n5) {
            return 0;
        }
        if (n < n3 && n < n5) {
            return (n4 < n6) ? 1 : -1;
        }
        if (n >= n3 + (n2 - n4) * (n5 - n3) / (n6 - n4)) {
            return 0;
        }
        return (n4 < n6) ? 1 : -1;
    }
    
    public static int pointCrossingsForQuad(double n, double n2, double n3, double n4, double v, double v2, double n5, double n6, int n7) {
        if (n2 < n4 && n2 < v2 && n2 < n6) {
            return 0;
        }
        if (n2 >= n4 && n2 >= v2 && n2 >= n6) {
            return 0;
        }
        if (n >= n3 && n >= v && n >= n5) {
            return 0;
        }
        if (n < n3 && n < v && n < n5) {
            if (n2 >= n4) {
                if (n2 < n6) {
                    return 1;
                }
            }
            else if (n2 >= n6) {
                return -1;
            }
            return 0;
        }
        else {
            if (n7 > 52) {
                return pointCrossingsForLine(n, n2, n3, n4, n5, n6);
            }
            double n8 = (n3 + v) / 2.0;
            double n9 = (n4 + v2) / 2.0;
            double n10 = (v + n5) / 2.0;
            double n11 = (v2 + n6) / 2.0;
            v = (n8 + n10) / 2.0;
            v2 = (n9 + n11) / 2.0;
            if (Double.isNaN(v) || Double.isNaN(v2)) {
                return 0;
            }
            return pointCrossingsForQuad(n, n2, n3, n4, n8, n9, v, v2, n7 + 1) + pointCrossingsForQuad(n, n2, v, v2, n10, n11, n5, n6, n7 + 1);
        }
    }
    
    public static int pointCrossingsForCubic(double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, int n11) {
        if (n2 < n4 && n2 < n6 && n2 < n8 && n2 < n10) {
            return 0;
        }
        if (n2 >= n4 && n2 >= n6 && n2 >= n8 && n2 >= n10) {
            return 0;
        }
        if (n >= n3 && n >= n5 && n >= n7 && n >= n9) {
            return 0;
        }
        if (n < n3 && n < n5 && n < n7 && n < n9) {
            if (n2 >= n4) {
                if (n2 < n10) {
                    return 1;
                }
            }
            else if (n2 >= n10) {
                return -1;
            }
            return 0;
        }
        else {
            if (n11 > 52) {
                return pointCrossingsForLine(n, n2, n3, n4, n9, n10);
            }
            double n12 = (n5 + n7) / 2.0;
            double n13 = (n6 + n8) / 2.0;
            n5 = (n3 + n5) / 2.0;
            n6 = (n4 + n6) / 2.0;
            n7 = (n7 + n9) / 2.0;
            n8 = (n8 + n10) / 2.0;
            double n14 = (n5 + n12) / 2.0;
            double n15 = (n6 + n13) / 2.0;
            double n16 = (n12 + n7) / 2.0;
            double n17 = (n13 + n8) / 2.0;
            double v = (n14 + n16) / 2.0;
            double v2 = (n15 + n17) / 2.0;
            if (Double.isNaN(v) || Double.isNaN(v2)) {
                return 0;
            }
            return pointCrossingsForCubic(n, n2, n3, n4, n5, n6, n14, n15, v, v2, n11 + 1) + pointCrossingsForCubic(n, n2, v, v2, n16, n17, n7, n8, n9, n10, n11 + 1);
        }
    }
    
    public static int rectCrossingsForPath(PathIterator pathIterator, double n, double n2, double n3, double n4) {
        if (n3 <= n || n4 <= n2) {
            return 0;
        }
        if (pathIterator.isDone()) {
            return 0;
        }
        double[] array = new double[6];
        if (pathIterator.currentSegment(array) != 0) {
            throw new IllegalPathStateException("missing initial moveto in path definition");
        }
        pathIterator.next();
        double n6;
        double n5 = n6 = array[0];
        double n8;
        double n7 = n8 = array[1];
        int n9 = 0;
        while (n9 != Integer.MIN_VALUE && !pathIterator.isDone()) {
            switch (pathIterator.currentSegment(array)) {
                case 0: {
                    if (n6 != n5 || n8 != n7) {
                        n9 = rectCrossingsForLine(n9, n, n2, n3, n4, n6, n8, n5, n7);
                    }
                    n6 = (n5 = array[0]);
                    n8 = (n7 = array[1]);
                    break;
                }
                case 1: {
                    double n10 = array[0];
                    double n11 = array[1];
                    n9 = rectCrossingsForLine(n9, n, n2, n3, n4, n6, n8, n10, n11);
                    n6 = n10;
                    n8 = n11;
                    break;
                }
                case 2: {
                    double n12 = array[2];
                    double n13 = array[3];
                    n9 = rectCrossingsForQuad(n9, n, n2, n3, n4, n6, n8, array[0], array[1], n12, n13, 0);
                    n6 = n12;
                    n8 = n13;
                    break;
                }
                case 3: {
                    double n14 = array[4];
                    double n15 = array[5];
                    n9 = rectCrossingsForCubic(n9, n, n2, n3, n4, n6, n8, array[0], array[1], array[2], array[3], n14, n15, 0);
                    n6 = n14;
                    n8 = n15;
                    break;
                }
                case 4: {
                    if (n6 != n5 || n8 != n7) {
                        n9 = rectCrossingsForLine(n9, n, n2, n3, n4, n6, n8, n5, n7);
                    }
                    n6 = n5;
                    n8 = n7;
                    break;
                }
            }
            pathIterator.next();
        }
        if (n9 != Integer.MIN_VALUE && (n6 != n5 || n8 != n7)) {
            n9 = rectCrossingsForLine(n9, n, n2, n3, n4, n6, n8, n5, n7);
        }
        return n9;
    }
    
    public static int rectCrossingsForLine(int n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9) {
        if (n7 >= n5 && n9 >= n5) {
            return n;
        }
        if (n7 <= n3 && n9 <= n3) {
            return n;
        }
        if (n6 <= n2 && n8 <= n2) {
            return n;
        }
        if (n6 >= n4 && n8 >= n4) {
            if (n7 < n9) {
                if (n7 <= n3) {
                    ++n;
                }
                if (n9 >= n5) {
                    ++n;
                }
            }
            else if (n9 < n7) {
                if (n9 <= n3) {
                    --n;
                }
                if (n7 >= n5) {
                    --n;
                }
            }
            return n;
        }
        else {
            if ((n6 > n2 && n6 < n4 && n7 > n3 && n7 < n5) || (n8 > n2 && n8 < n4 && n9 > n3 && n9 < n5)) {
                return Integer.MIN_VALUE;
            }
            if (n7 < n3) {
                n6 += (n3 - n7) * (n8 - n6) / (n9 - n7);
            }
            else if (n7 > n5) {
                n6 += (n5 - n7) * (n8 - n6) / (n9 - n7);
            }
            if (n9 < n3) {
                n8 += (n3 - n9) * (n6 - n8) / (n7 - n9);
            }
            else if (n9 > n5) {
                n8 += (n5 - n9) * (n6 - n8) / (n7 - n9);
            }
            if (n6 <= n2 && n8 <= n2) {
                return n;
            }
            if (n6 >= n4 && n8 >= n4) {
                if (n7 < n9) {
                    if (n7 <= n3) {
                        ++n;
                    }
                    if (n9 >= n5) {
                        ++n;
                    }
                }
                else if (n9 < n7) {
                    if (n9 <= n3) {
                        --n;
                    }
                    if (n7 >= n5) {
                        --n;
                    }
                }
                return n;
            }
            else {
                return Integer.MIN_VALUE;
            }
        }
    }
    
    public static int rectCrossingsForQuad(int n, double n2, double n3, double n4, double n5, double n6, double n7, double v, double v2, double n8, double n9, int n10) {
        if (n7 >= n5 && v2 >= n5 && n9 >= n5) {
            return n;
        }
        if (n7 <= n3 && v2 <= n3 && n9 <= n3) {
            return n;
        }
        if (n6 <= n2 && v <= n2 && n8 <= n2) {
            return n;
        }
        if (n6 >= n4 && v >= n4 && n8 >= n4) {
            if (n7 < n9) {
                if (n7 <= n3 && n9 > n3) {
                    ++n;
                }
                if (n7 < n5 && n9 >= n5) {
                    ++n;
                }
            }
            else if (n9 < n7) {
                if (n9 <= n3 && n7 > n3) {
                    --n;
                }
                if (n9 < n5 && n7 >= n5) {
                    --n;
                }
            }
            return n;
        }
        else {
            if ((n6 < n4 && n6 > n2 && n7 < n5 && n7 > n3) || (n8 < n4 && n8 > n2 && n9 < n5 && n9 > n3)) {
                return Integer.MIN_VALUE;
            }
            if (n10 > 52) {
                return rectCrossingsForLine(n, n2, n3, n4, n5, n6, n7, n8, n9);
            }
            double n11 = (n6 + v) / 2.0;
            double n12 = (n7 + v2) / 2.0;
            double n13 = (v + n8) / 2.0;
            double n14 = (v2 + n9) / 2.0;
            v = (n11 + n13) / 2.0;
            v2 = (n12 + n14) / 2.0;
            if (Double.isNaN(v) || Double.isNaN(v2)) {
                return 0;
            }
            n = rectCrossingsForQuad(n, n2, n3, n4, n5, n6, n7, n11, n12, v, v2, n10 + 1);
            if (n != Integer.MIN_VALUE) {
                n = rectCrossingsForQuad(n, n2, n3, n4, n5, v, v2, n13, n14, n8, n9, n10 + 1);
            }
            return n;
        }
    }
    
    public static int rectCrossingsForCubic(int n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12, double n13, int n14) {
        if (n7 >= n5 && n9 >= n5 && n11 >= n5 && n13 >= n5) {
            return n;
        }
        if (n7 <= n3 && n9 <= n3 && n11 <= n3 && n13 <= n3) {
            return n;
        }
        if (n6 <= n2 && n8 <= n2 && n10 <= n2 && n12 <= n2) {
            return n;
        }
        if (n6 >= n4 && n8 >= n4 && n10 >= n4 && n12 >= n4) {
            if (n7 < n13) {
                if (n7 <= n3 && n13 > n3) {
                    ++n;
                }
                if (n7 < n5 && n13 >= n5) {
                    ++n;
                }
            }
            else if (n13 < n7) {
                if (n13 <= n3 && n7 > n3) {
                    --n;
                }
                if (n13 < n5 && n7 >= n5) {
                    --n;
                }
            }
            return n;
        }
        else {
            if ((n6 > n2 && n6 < n4 && n7 > n3 && n7 < n5) || (n12 > n2 && n12 < n4 && n13 > n3 && n13 < n5)) {
                return Integer.MIN_VALUE;
            }
            if (n14 > 52) {
                return rectCrossingsForLine(n, n2, n3, n4, n5, n6, n7, n12, n13);
            }
            double n15 = (n8 + n10) / 2.0;
            double n16 = (n9 + n11) / 2.0;
            n8 = (n6 + n8) / 2.0;
            n9 = (n7 + n9) / 2.0;
            n10 = (n10 + n12) / 2.0;
            n11 = (n11 + n13) / 2.0;
            double n17 = (n8 + n15) / 2.0;
            double n18 = (n9 + n16) / 2.0;
            double n19 = (n15 + n10) / 2.0;
            double n20 = (n16 + n11) / 2.0;
            double v = (n17 + n19) / 2.0;
            double v2 = (n18 + n20) / 2.0;
            if (Double.isNaN(v) || Double.isNaN(v2)) {
                return 0;
            }
            n = rectCrossingsForCubic(n, n2, n3, n4, n5, n6, n7, n8, n9, n17, n18, v, v2, n14 + 1);
            if (n != Integer.MIN_VALUE) {
                n = rectCrossingsForCubic(n, n2, n3, n4, n5, v, v2, n19, n20, n10, n11, n12, n13, n14 + 1);
            }
            return n;
        }
    }
    
    public Curve(int direction) {
        this.direction = direction;
    }
    
    public final int getDirection() {
        return this.direction;
    }
    
    public final Curve getWithDirection(int n) {
        return (this.direction == n) ? this : this.getReversedCurve();
    }
    
    public static double round(double n) {
        return n;
    }
    
    public static int orderof(double n, double n2) {
        if (n < n2) {
            return -1;
        }
        if (n > n2) {
            return 1;
        }
        return 0;
    }
    
    public static long signeddiffbits(double value, double value2) {
        return Double.doubleToLongBits(value) - Double.doubleToLongBits(value2);
    }
    
    public static long diffbits(double value, double value2) {
        return Math.abs(Double.doubleToLongBits(value) - Double.doubleToLongBits(value2));
    }
    
    public static double prev(double value) {
        return Double.longBitsToDouble(Double.doubleToLongBits(value) - 1L);
    }
    
    public static double next(double value) {
        return Double.longBitsToDouble(Double.doubleToLongBits(value) + 1L);
    }
    
    @Override
    public String toString() {
        return "Curve[" + this.getOrder() + ", " + "(" + round(this.getX0()) + ", " + round(this.getY0()) + "), " + this.controlPointString() + "(" + round(this.getX1()) + ", " + round(this.getY1()) + "), " + ((this.direction == 1) ? "D" : "U") + "]";
    }
    
    public String controlPointString() {
        return "";
    }
    
    public abstract int getOrder();
    
    public abstract double getXTop();
    
    public abstract double getYTop();
    
    public abstract double getXBot();
    
    public abstract double getYBot();
    
    public abstract double getXMin();
    
    public abstract double getXMax();
    
    public abstract double getX0();
    
    public abstract double getY0();
    
    public abstract double getX1();
    
    public abstract double getY1();
    
    public abstract double XforY(double p0);
    
    public abstract double TforY(double p0);
    
    public abstract double XforT(double p0);
    
    public abstract double YforT(double p0);
    
    public abstract double dXforT(double p0, int p1);
    
    public abstract double dYforT(double p0, int p1);
    
    public abstract double nextVertical(double p0, double p1);
    
    public int crossingsFor(double n, double n2) {
        if (n2 >= this.getYTop() && n2 < this.getYBot() && n < this.getXMax() && (n < this.getXMin() || n < this.XforY(n2))) {
            return 1;
        }
        return 0;
    }
    
    public boolean accumulateCrossings(Crossings crossings) {
        double xHi = crossings.getXHi();
        if (this.getXMin() >= xHi) {
            return false;
        }
        double xLo = crossings.getXLo();
        double yLo = crossings.getYLo();
        double yHi = crossings.getYHi();
        double yTop = this.getYTop();
        double yBot = this.getYBot();
        double n;
        if (yTop < yLo) {
            if (yBot <= yLo) {
                return false;
            }
            n = this.TforY(yLo);
        }
        else {
            if (yTop >= yHi) {
                return false;
            }
            yLo = yTop;
            n = 0.0;
        }
        double tforY;
        if (yBot > yHi) {
            tforY = this.TforY(yHi);
        }
        else {
            yHi = yBot;
            tforY = 1.0;
        }
        boolean n2 = false;
        boolean n3 = false;
        while (true) {
            double xforT = this.XforT(n);
            if (xforT < xHi) {
                if (n3 || xforT > xLo) {
                    return true;
                }
                n2 = true;
            }
            else {
                if (!(n2 == 0)) {
                    return true;
                }
                n3 = true;
            }
            if (n >= tforY) {
                if (!(n2 == 0)) {
                    crossings.record(yLo, yHi, this.direction);
                }
                return false;
            }
            else {
                n = this.nextVertical(n, tforY);
            }
        }
    }
    
    public abstract void enlarge(Rectangle2D p0);
    
    public Curve getSubCurve(double n, double n2) {
        return this.getSubCurve(n, n2, this.direction);
    }
    
    public abstract Curve getReversedCurve();
    
    public abstract Curve getSubCurve(double p0, double p1, int p2);
    
    public int compareTo(Curve obj, double[] array) {
        double a = array[0];
        double min = Math.min(Math.min(array[1], this.getYBot()), obj.getYBot());
        if (min <= array[0]) {
            System.err.println("this == " + this);
            System.err.println("that == " + obj);
            System.out.println("target range = " + array[0] + "=>" + array[1]);
            throw new InternalError("backstepping from " + array[0] + " to " + min);
        }
        array[1] = min;
        if (this.getXMax() <= obj.getXMin()) {
            if (this.getXMin() == obj.getXMax()) {
                return 0;
            }
            return -1;
        }
        else {
            if (this.getXMin() >= obj.getXMax()) {
                return 1;
            }
            double n = this.TforY(a);
            double d = this.YforT(n);
            if (d < a) {
                n = this.refineTforY(n, d, a);
                d = this.YforT(n);
            }
            double n2 = this.TforY(min);
            if (this.YforT(n2) < a) {
                n2 = this.refineTforY(n2, this.YforT(n2), a);
            }
            double n3 = obj.TforY(a);
            double d2 = obj.YforT(n3);
            if (d2 < a) {
                n3 = obj.refineTforY(n3, d2, a);
                d2 = obj.YforT(n3);
            }
            double n4 = obj.TforY(min);
            if (obj.YforT(n4) < a) {
                n4 = obj.refineTforY(n4, obj.YforT(n4), a);
            }
            double xforT = this.XforT(n);
            double xforT2 = obj.XforT(n3);
            double max = Math.max(Math.max(Math.abs(a), Math.abs(min)) * 1.0E-14, 1.0E-300);
            if (this.fairlyClose(xforT, xforT2)) {
                double n5 = max;
                double min2 = Math.min(max * 1.0E13, (min - a) * 0.1);
                double n6 = a + n5;
                while (n6 <= min) {
                    if (this.fairlyClose(this.XforY(n6), obj.XforY(n6))) {
                        if ((n5 *= 2.0) > min2) {
                            n5 = min2;
                        }
                        n6 += n5;
                    }
                    else {
                        n6 -= n5;
                        while (true) {
                            n5 /= 2.0;
                            double n7 = n6 + n5;
                            if (n7 <= n6) {
                                break;
                            }
                            if (this.fairlyClose(this.XforY(n7), obj.XforY(n7))) {
                                n6 = n7;
                            }
                            else {
                                continue;
                            }
                        }
                        break;
                    }
                }
                if (n6 > a) {
                    if (n6 < min) {
                        array[1] = n6;
                    }
                    return 0;
                }
            }
            if (max <= 0.0) {
                System.out.println("ymin = " + max);
            }
            while (n < n2 && n3 < n4) {
                double nextVertical = this.nextVertical(n, n2);
                double xforT3 = this.XforT(nextVertical);
                double yforT = this.YforT(nextVertical);
                double nextVertical2 = obj.nextVertical(n3, n4);
                double xforT4 = obj.XforT(nextVertical2);
                double yforT2 = obj.YforT(nextVertical2);
                try {
                    if (this.findIntersect(obj, array, max, 0, 0, n, xforT, d, nextVertical, xforT3, yforT, n3, xforT2, d2, nextVertical2, xforT4, yforT2)) {
                        break;
                    }
                }
                catch (Throwable obj2) {
                    System.err.println("Error: " + obj2);
                    System.err.println("y range was " + array[0] + "=>" + array[1]);
                    System.err.println("s y range is " + d + "=>" + yforT);
                    System.err.println("t y range is " + d2 + "=>" + yforT2);
                    System.err.println("ymin is " + max);
                    return 0;
                }
                if (yforT < yforT2) {
                    if (yforT > array[0]) {
                        if (yforT < array[1]) {
                            array[1] = yforT;
                            break;
                        }
                        break;
                    }
                    else {
                        n = nextVertical;
                        xforT = xforT3;
                        d = yforT;
                    }
                }
                else if (yforT2 > array[0]) {
                    if (yforT2 < array[1]) {
                        array[1] = yforT2;
                        break;
                    }
                    break;
                }
                else {
                    n3 = nextVertical2;
                    xforT2 = xforT4;
                    d2 = yforT2;
                }
            }
            double n8 = (array[0] + array[1]) / 2.0;
            return orderof(this.XforY(n8), obj.XforY(n8));
        }
    }
    
    public boolean findIntersect(Curve curve, double[] array, double n, int n2, int n3, double d, double n4, double n5, double d2, double n6, double n7, double n8, double n9, double n10, double n11, double n12, double n13) {
        if (n5 > n13 || n10 > n7) {
            return false;
        }
        if (Math.min(n4, n6) > Math.max(n9, n12) || Math.max(n4, n6) < Math.min(n9, n12)) {
            return false;
        }
        if (d2 - d > 0.001) {
            double n28 = (d + d2) / 2.0;
            double xforT2 = this.XforT(n28);
            double yforT2 = this.YforT(n28);
            if (n28 == d || n28 == d2) {
                System.out.println("s0 = " + d);
                System.out.println("s1 = " + d2);
                throw new InternalError("no s progress!");
            }
            if (n11 - n8 > 0.001) {
                double n29 = (n8 + n11) / 2.0;
                double xforT3 = curve.XforT(n29);
                double yforT3 = curve.YforT(n29);
                if (n29 == n8 || n29 == n11) {
                    System.out.println("t0 = " + n8);
                    System.out.println("t1 = " + n11);
                    throw new InternalError("no t progress!");
                }
                if (yforT2 >= n10 && yforT3 >= n5 && this.findIntersect(curve, array, n, n2 + 1, n3 + 1, d, n4, n5, n28, xforT2, yforT2, n8, n9, n10, n29, xforT3, yforT3)) {
                    return true;
                }
                if (yforT2 >= yforT3 && this.findIntersect(curve, array, n, n2 + 1, n3 + 1, d, n4, n5, n28, xforT2, yforT2, n29, xforT3, yforT3, n11, n12, n13)) {
                    return true;
                }
                if (yforT3 >= yforT2 && this.findIntersect(curve, array, n, n2 + 1, n3 + 1, n28, xforT2, yforT2, d2, n6, n7, n8, n9, n10, n29, xforT3, yforT3)) {
                    return true;
                }
                if (n7 >= yforT3 && n13 >= yforT2 && this.findIntersect(curve, array, n, n2 + 1, n3 + 1, n28, xforT2, yforT2, d2, n6, n7, n29, xforT3, yforT3, n11, n12, n13)) {
                    return true;
                }
            }
            else {
                if (yforT2 >= n10 && this.findIntersect(curve, array, n, n2 + 1, n3, d, n4, n5, n28, xforT2, yforT2, n8, n9, n10, n11, n12, n13)) {
                    return true;
                }
                if (n13 >= yforT2 && this.findIntersect(curve, array, n, n2 + 1, n3, n28, xforT2, yforT2, d2, n6, n7, n8, n9, n10, n11, n12, n13)) {
                    return true;
                }
            }
        }
        else if (n11 - n8 > 0.001) {
            double n27 = (n8 + n11) / 2.0;
            double xforT = curve.XforT(n27);
            double yforT = curve.YforT(n27);
            if (n27 == n8 || n27 == n11) {
                System.out.println("t0 = " + n8);
                System.out.println("t1 = " + n11);
                throw new InternalError("no t progress!");
            }
            if (yforT >= n5 && this.findIntersect(curve, array, n, n2, n3 + 1, d, n4, n5, d2, n6, n7, n8, n9, n10, n27, xforT, yforT)) {
                return true;
            }
            if (n7 >= yforT && this.findIntersect(curve, array, n, n2, n3 + 1, d, n4, n5, d2, n6, n7, n27, xforT, yforT, n11, n12, n13)) {
                return true;
            }
        }
        else {
            double n14 = n6 - n4;
            double n15 = n7 - n5;
            double n16 = n12 - n9;
            double n17 = n13 - n10;
            double n18 = n9 - n4;
            double n19 = n10 - n5;
            double n20 = n16 * n15 - n17 * n14;
            if (n20 != 0.0) {
                double n21 = 1.0 / n20;
                double n22 = (n16 * n19 - n17 * n18) * n21;
                double n23 = (n14 * n19 - n15 * n18) * n21;
                if (n22 >= 0.0 && n22 <= 1.0 && n23 >= 0.0 && n23 <= 1.0) {
                    double n24 = d + n22 * (d2 - d);
                    double n25 = n8 + n23 * (n11 - n8);
                    if (n24 < 0.0 || n24 > 1.0 || n25 < 0.0 || n25 > 1.0) {
                        System.out.println("Uh oh!");
                    }
                    double n26 = (this.YforT(n24) + curve.YforT(n25)) / 2.0;
                    if (n26 <= array[1] && n26 > array[0]) {
                        array[1] = n26;
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public double refineTforY(double n, double n2, double n3) {
        double n4 = 1.0;
        while (true) {
            double n5 = (n + n4) / 2.0;
            if (n5 == n || n5 == n4) {
                return n4;
            }
            double yforT = this.YforT(n5);
            if (yforT < n3) {
                n = n5;
                n2 = yforT;
            }
            else if (yforT > n3) {
                n4 = n5;
            }
            else {
                return n4;
            }
        }
    }
    
    public boolean fairlyClose(double a, double a2) {
        return Math.abs(a - a2) < Math.max(Math.abs(a), Math.abs(a2)) * 1.0E-10;
    }
    
    public abstract int getSegment(double[] p0);
}
